/*
  Copyright (c) 2017, Lawrence Livermore National Security, LLC.
  Produced at the Lawrence Livermore National Laboratory
  Written by Chunhua Liao, Pei-Hung Lin, Joshua Asplund,
  Markus Schordan, and Ian Karlin
  (email: liao6@llnl.gov, lin32@llnl.gov, asplund1@llnl.gov,
  schordan1@llnl.gov, karlin1@llnl.gov)
  LLNL-CODE-732144
  All rights reserved.

  This file is part of DataRaceBench. For details, see
  https://github.com/LLNL/dataracebench. Please also see the LICENSE file
  for our additional BSD notice.

  Redistribution and use in source and binary forms, with
  or without modification, are permitted provided that the following
  conditions are met:

  * Redistributions of source code must retain the above copyright
  notice, this list of conditions and the disclaimer below.

  * Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the disclaimer (as noted below)
  in the documentation and/or other materials provided with the
  distribution.

  * Neither the name of the LLNS/LLNL nor the names of its contributors
  may be used to endorse or promote products derived from this
  software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL LAWRENCE LIVERMORE NATIONAL
  SECURITY, LLC, THE U.S. DEPARTMENT OF ENERGY OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
  The -- operation on numNodes2 is not protected, causing data race.
  Data race pair: numNodes2@74:7 vs. numNodes2@74:7
*/
#include <stdio.h>
#include <omp.h>
int main(int argc, char* argv[])
{
  omp_set_num_threads(2);

  const int len = 100;
  int x[len];
  int numNodes = len, numNodes2 = 0;

  for (int i=0; i<len; i++) {
    if (i%2 == 0) {
      x[i] = 5;
    } else {
      x[i] = -5;
    }
  }

#pragma omp parallel for
  for (int i=numNodes-1; i > -1; --i) {
    if (x[i] <= 0) {
      numNodes2-- ;
    }
  }

  if (numNodes2 != -numNodes/2) {
    printf ("numNodes2 = %d\n", numNodes2);
    printf ("A race occured\n");
  } else {
    printf ("No race found\n");
  }

  return 0;
}
